package stdune

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Console.BackendSource

Sourcetype t = (module Backend)
Sourceval set : t -> unit
Sourceval compose : t -> t -> t

compose a b produce a backend that sends message to both a and b backends.

Sourceval dumb : t

A dumb backend that hides the status line and simply dumps the messages to the terminal.

Sourceval progress : t

A backend that displays the status line in the terminal.

Sourceval flush : t -> t

flush t returns a backend that will flush stderr after every write

Sourceval progress_no_flush : t

A base progress backend that doesn't flush. Any backend implemented on top if is expected to flush manually