package stdune

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Stdune.CharSource

include module type of struct include Char end

Characters

Sourcetype t = char

An alias for the type of characters.

Sourceval code : char -> int

Return the integer code of the argument.

Sourceval chr : int -> char

Return the character with the given integer code.

Sourceval escaped : char -> string

Return a string representing the given character, with special characters escaped following the lexical conventions of OCaml. All characters outside the ASCII printable range [0x20;0x7E] are escaped, as well as backslash, double-quote, and single-quote.

Predicates and comparisons

See also the Ascii module.

Sourceval equal : t -> t -> bool

The equal function for chars.

  • since 4.03

ASCII characters

Sourcemodule Ascii = Char.Ascii

ASCII character set support.

Sourceval lowercase_ascii : char -> char

Use the equivalent Ascii.lowercase instead.

  • since 4.03
Sourceval uppercase_ascii : char -> char

Use the equivalent Ascii.uppercase instead.

  • since 4.03

Hashing

Sourceval seeded_hash : int -> t -> int

A seeded hash function for characters, with the same output value as Hashtbl.seeded_hash. This function allows this module to be passed as argument to the functor Hashtbl.MakeSeeded.

  • since 5.1
Sourceval is_digit : t -> bool

Check if a character belongs to the set {'0'..'9'}.

Sourceval is_lowercase_hex : t -> bool

Check if a character belongs to the set {'0'..'9', 'a'..'f'}.

Sourceval hash : t -> int
Sourceval compare : t -> t -> Ordering.t