package stdune

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Stdune.ProcSource

Sourceval restore_cwd_and_execve : string -> string list -> env:Env.t -> _

Proc.restore_cwd_and_execve prog args ~env runs prog with args in env.

  • prog is the program being run. It should be a filename in the current working directory.
  • args is a list of arguments to the program. Unlike in the system call execve, the first argument is not the program name. The first argument is the first argument to the program. The program name is set to prog without the caller needing to.
  • env is the environment in which the program is run.
Sourcemodule Resource_usage : sig ... end
Sourcemodule Times : sig ... end
Sourcemodule Process_info : sig ... end
Sourcetype wait =
  1. | Any
  2. | Pid of Pid.t
Sourceval wait : wait -> Unix.wait_flag list -> Process_info.t option

This function is not implemented on Windows.

Returns None if there are no children. If WNOHANG is passed, also returns None if none of the proceseses are finished yet. When successful, returns information about the reaped process.